package request

import (
	"git.sxidc.com/go-framework/baize/framework/core/api"
	"github.com/gin-gonic/gin/binding"
	"github.com/pkg/errors"
)

// BindRequestParamsFunc 请求绑定函数
type BindRequestParamsFunc func(c *api.Context, params Params) error

// BindJsonBody 绑定Json类型的Body请求到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindJsonBody(c *api.Context, params Params) error {
	err := c.ShouldBindJSON(params)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}

// BindQueryParams 绑定查询参数到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindQueryParams(c *api.Context, params Params) error {
	err := c.ShouldBindQuery(params)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}

// BindPathParams 绑定路径参数到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindPathParams(c *api.Context, params Params) error {
	err := c.ShouldBindUri(params)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}

// BindMultipartForm 绑定multipart body到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindMultipartForm(c *api.Context, params Params) error {
	err := c.ShouldBindWith(params, binding.FormMultipart)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}

// BindFormBody 绑定form body到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindFormBody(c *api.Context, params Params) error {
	err := c.ShouldBindWith(params, binding.Form)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}

// BindXMLBody 绑定xml body到请求参数接口
// 参数:
// - c: 上下文
// - params: 请求参数接口
// 返回值:
// - 错误
func BindXMLBody(c *api.Context, params Params) error {
	err := c.ShouldBindXML(params)
	if err != nil {
		return errors.New(err.Error())
	}

	return nil
}
